<?php
namespace Qiniu\Processing;

use Qiniu;

/**
 * Ҫ漰ͼƬƴ
 *
 * @link http://developer.qiniu.com/code/v6/api/kodo-api/image/imageview2.html
 */
final class ImageUrlBuilder
{
    /**
     * modeϷΧֵ
     *
     * @var array
     */
    protected $modeArr = array(0, 1, 2, 3, 4, 5);

    /**
     * formatϷֵ
     *
     * @var array
     */
    protected $formatArr = array('psd', 'jpeg', 'png', 'gif', 'webp', 'tiff', 'bmp');

    /**
     * ˮӡͼƬλúϷֵ
     *
     * @var array
     */
    protected $gravityArr = array('NorthWest', 'North', 'NorthEast',
        'West', 'Center', 'East', 'SouthWest', 'South', 'SouthEast');

    /**
     * ͼƴ
     *
     * @param  string $url ͼƬ
     * @param  int $mode ģʽ
     * @param  int $width 
     * @param  int $height 
     * @param  string $format 
     * @param  int $quality ͼƬ
     * @param  int $interlace Ƿֽ֧ʾ
     * @param  int $ignoreError Խ
     * @return string
     * @link http://developer.qiniu.com/code/v6/api/kodo-api/image/imageview2.html
     * @author Sherlock Ren <sherlock_ren@icloud.com>
     */
    public function thumbnail(
        $url,
        $mode,
        $width,
        $height,
        $format = null,
        $interlace = null,
        $quality = null,
        $ignoreError = 1
    ) {
    
        // urlϷЧ
        if (! $this->isUrl($url)) {
            return $url;
        }

        // ϷЧ
        if (! in_array(intval($mode), $this->modeArr, true)) {
            return $url;
        }

        if (! $width || ! $height) {
            return $url;
        }

        $thumbStr = 'imageView2/' . $mode . '/w/' . $width . '/h/' . $height . '/';

        // ƴʽ
        if (! is_null($format)
            && in_array($format, $this->formatArr)
        ) {
            $thumbStr .= 'format/' . $format . '/';
        }

        // ƴӽʾ
        if (! is_null($interlace)
            && in_array(intval($interlace), array(0, 1), true)
        ) {
            $thumbStr .= 'interlace/' . $interlace . '/';
        }

        // ƴͼƬ
        if (! is_null($quality)
            && intval($quality) >= 0
            && intval($quality) <= 100
        ) {
            $thumbStr .= 'q/' . $quality . '/';
        }

        $thumbStr .= 'ignore-error/' . $ignoreError . '/';

        // query_string|߷ָʵֶ
        return $url . ($this->hasQuery($url) ? '|' : '?') . $thumbStr;
    }

    /**
     * ͼƬˮӡ
     *
     * @param  string $url ͼƬ
     * @param  string $image ˮӡͼƬ
     * @param  numeric $dissolve ͸
     * @param  string $gravity ˮӡλ
     * @param  numeric $dx ߾
     * @param  numeric $dy ߾
     * @param  numeric $watermarkScale ӦԭͼĶ̱߱
     * @link   http://developer.qiniu.com/code/v6/api/kodo-api/image/watermark.html
     * @return string
     * @author Sherlock Ren <sherlock_ren@icloud.com>
     */
    public function waterImg(
        $url,
        $image,
        $dissolve = 100,
        $gravity = 'SouthEast',
        $dx = null,
        $dy = null,
        $watermarkScale = null
    ) {
        // urlϷЧ
        if (! $this->isUrl($url)) {
            return $url;
        }

        $waterStr = 'watermark/1/image/' . \Qiniu\base64_urlSafeEncode($image) . '/';

        // ƴˮӡ͸
        if (is_numeric($dissolve)
            && $dissolve <= 100
        ) {
            $waterStr .= 'dissolve/' . $dissolve . '/';
        }

        // ƴˮӡλ
        if (in_array($gravity, $this->gravityArr, true)) {
            $waterStr .= 'gravity/' . $gravity . '/';
        }

        // ƴӺ߾
        if (! is_null($dx)
            && is_numeric($dx)
        ) {
            $waterStr .= 'dx/' . $dx . '/';
        }

        // ƴ߾
        if (! is_null($dy)
            && is_numeric($dy)
        ) {
            $waterStr .= 'dy/' . $dy . '/';
        }

        // ƴӦԭͼĶ̱߱
        if (! is_null($watermarkScale)
            && is_numeric($watermarkScale)
            && $watermarkScale > 0
            && $watermarkScale < 1
        ) {
            $waterStr .= 'ws/' . $watermarkScale . '/';
        }

        // query_string|߷ָʵֶ
        return $url . ($this->hasQuery($url) ? '|' : '?') . $waterStr;
    }

    /**
     * ˮӡ
     *
     * @param  string $url ͼƬ
     * @param  string $text 
     * @param  string $font 
     * @param  string $fontSize ֺ
     * @param  string $fontColor ɫ
     * @param  numeric $dissolve ͸
     * @param  string $gravity ˮӡλ
     * @param  numeric $dx ߾
     * @param  numeric $dy ߾
     * @link   http://developer.qiniu.com/code/v6/api/kodo-api/image/watermark.html#text-watermark
     * @return string
     * @author Sherlock Ren <sherlock_ren@icloud.com>
     */
    public function waterText(
        $url,
        $text,
        $font = '',
        $fontSize = 0,
        $fontColor = null,
        $dissolve = 100,
        $gravity = 'SouthEast',
        $dx = null,
        $dy = null
    ) {
        // urlϷЧ
        if (! $this->isUrl($url)) {
            return $url;
        }

        $waterStr = 'watermark/2/text/'
            . \Qiniu\base64_urlSafeEncode($text) . '/font/'
            . \Qiniu\base64_urlSafeEncode($font) . '/';

        // ƴִС
        if (is_int($fontSize)) {
            $waterStr .= 'fontsize/' . $fontSize . '/';
        }

        // ƴɫ
        if (! is_null($fontColor)
            && $fontColor
        ) {
            $waterStr .= 'fill/' . \Qiniu\base64_urlSafeEncode($fontColor) . '/';
        }

        // ƴˮӡ͸
        if (is_numeric($dissolve)
            && $dissolve <= 100
        ) {
            $waterStr .= 'dissolve/' . $dissolve . '/';
        }

        // ƴˮӡλ
        if (in_array($gravity, $this->gravityArr, true)) {
            $waterStr .= 'gravity/' . $gravity . '/';
        }

        // ƴӺ߾
        if (! is_null($dx)
            && is_numeric($dx)
        ) {
            $waterStr .= 'dx/' . $dx . '/';
        }

        // ƴ߾
        if (! is_null($dy)
            && is_numeric($dy)
        ) {
            $waterStr .= 'dy/' . $dy . '/';
        }

        // query_string|߷ָʵֶ
        return $url . ($this->hasQuery($url) ? '|' : '?') . $waterStr;
    }

    /**
     * ЧurlϷ
     *
     * @param  string $url url
     * @return string
     * @author Sherlock Ren <sherlock_ren@icloud.com>
     */
    protected function isUrl($url)
    {
        $urlArr = parse_url($url);

        return $urlArr['scheme']
            && in_array($urlArr['scheme'], array('http', 'https'))
            && $urlArr['host']
            && $urlArr['path'];
    }

    /**
     * Ƿquery
     *
     * @param  string $url url
     * @return string
     * @author Sherlock Ren <sherlock_ren@icloud.com>
     */
    protected function hasQuery($url)
    {
        $urlArr = parse_url($url);

        return ! empty($urlArr['query']);
    }
}
